@@ -2,9 +2,22 @@
 
 # If your distribution puts this in other locations, please adjust the values of
 # these variables before installing!
-INIT_D=/etc/init.d
+# INIT_D=/etc/init.d
 MAN_D=/usr/local/man/man8
 
+if [ "$INIT_D" == "" ] ; then
+	if [ -d /etc/rc.d/init.d ] ; then
+		INIT_D=/etc/rc.d/init.d
+	elif [ -d /etc/rc.d ] ; then
+		INIT_D=/etc/rc.d
+	elif [ -d /etc/init.d ] ; then
+		INIT_D=/etc/init.d
+	else
+		echo "Cannot determine location of init scripts. Please modify install.sh."
+		exit 31
+	fi
+fi
+
 if [ "`whoami`" != "root" ] ; then
 	echo "You need to be root to install the laptop mode tools."
 	exit 10
@@ -51,7 +64,7 @@ if ( ! $INSTALL -m 700 usr/sbin/lm-syslog-setup /usr/sbin ) ; then
 fi
 
 ACPI_DONE=0
-if [ -d /etc/acpi ] ; then
+if [ ! -d /proc/pmu -a -d /etc/acpi ] ; then
 	mkdir -p /etc/acpi/actions
 	mkdir -p /etc/acpi/events
 	
@@ -69,12 +82,13 @@ if [ -d /etc/acpi ] ; then
 		echo "$0: Failed to install ACPI event file in /etc/acpi/events. Installation failed."
 		exit 14
 	fi
+	killall -HUP acpid
 	echo "Installed ACPI support."
 	ACPI_DONE=1
 fi
 
 APM_DONE=0
-if [ -d /etc/apm ] ; then
+if [ ! -d /proc/pmu -a -d /etc/apm ] ; then
 	mkdir -p /etc/apm/event.d
 	if ( ! $INSTALL -m 700 etc/apm/event.d/* /etc/apm/event.d ) ; then
 		echo "$0: Failed to install APM event script in /etc/apm/event.d. Installation failed."
@@ -84,9 +98,43 @@ if [ -d /etc/apm ] ; then
 	APM_DONE=1
 fi
 
-if [ $APM_DONE -eq 0 -a $ACPI_DONE -eq 0 ] ; then
-	echo "Neither APM nor ACPI daemons were found. Laptop mode will not start automatically."
-	echo "Install either acpid or apmd (depending on what your laptop supports) and reinstall."
+PMU_DONE=0
+if [ -d /proc/pmu -a -d /etc/power ] ; then
+	mkdir -p /etc/power/event.d /etc/power/scripts.d
+	if ( ! $INSTALL -m 700 etc/power/scripts.d/laptop-mode /etc/power/scripts.d ) ; then
+		echo "$0: Failed to install pbbuttonsd event script in /etc/power/scripts.d. Installation failed."
+		exit 33
+	fi
+	if ( ! ln -fs ../scripts.d/laptop-mode /etc/power/event.d ) ; then
+		echo "$0: Failed to install pbbuttonsd event script in /etc/power/event.d. Installation failed."
+		exit 34
+	fi
+	if [ -f /etc/power/pwrctl ] ; then
+		if ( ! grep pwrctl-local /etc/power/pwrctl ) ; then
+			echo "WARNING: /etc/power/pwrctl does not call pwrctl-local. Laptop mode will not start automatically when you use pmud."
+		fi
+		touch /etc/power/pwrctl-local
+		if ( ! grep laptop_mode /etc/power/pwrctl-local ) ; then
+			(echo "#! /bin/bash" ; echo "/usr/sbin/laptop_mode auto" ; cat /etc/power/pwrctl-local) > /etc/power/pwrctl-local-tmp
+			cat /etc/power/pwrctl-local-tmp /etc/power/pwrctl-local
+			rm /etc/power/pwrctl-local-tmp
+		else
+			echo "/etc/power/pwrctl-local already seems to contain a laptop mode call. Not adding an extra one."
+		fi
+	fi
+	if [ -f /etc/apm/event.d/laptop-mode ] ; then
+		# This file interferes with the pbbuttonsd integration,
+		# because pbbuttonsd also emulates APM, so we have to
+		# remove it.
+		rm /etc/apm/event.d/laptop-mode
+	fi
+	echo "Installed PMU (pmud/pbbuttonsd) support."
+	PMU_DONE=1
+fi
+
+if [ $APM_DONE -eq 0 -a $ACPI_DONE -eq 0 -a $PMU_DONE -eq 0 ] ; then
+	echo "ACPI/APM/PMU support was not found. Laptop mode will not start automatically."
+	echo "Install either acpid, apmd, pbbuttonsd or pmud (depending on what your laptop supports) and reinstall."
 fi
 
 if [ -d $INIT_D ] ; then
@@ -98,12 +146,12 @@ if [ -d $INIT_D ] ; then
     # Old symlink.
     rm $RCS_D/S99laptop-mode
   fi
-  if [ -e "`which update-rc.d`" ] ; then
+  if ( which update-rc.d > /dev/null ) ; then
     if ( ! update-rc.d laptop-mode defaults ) ; then
       echo "$0: update-rc.d failed, laptop mode will not be initialized at bootup."
       exit 17
     fi
-  elif [ -e "`which chkconfig`" ] ; then
+  elif ( which chkconfig > /dev/null ) ; then
     if ( ! chkconfig laptop-mode on ) ; then
       echo "$0: chkconfig failed, laptop mode will not be initialized at bootup."
       exit 30
@@ -123,18 +171,18 @@ if ( ! cp man/* $MAN_D ) ; then
   exit 23
 fi
 
-if [ -e "`which invoke-rc.d`" ] ; then
+if ( which invoke-rc.d > /dev/null ) ; then
 	# Debian uses invoke-rc.d
 	RCPROG=invoke-rc.d
 	INITSCRIPT=laptop-mode
-elif [ -e "`which service`" ] ; then
+elif ( which service > /dev/null ) ; then
 	# RedHat uses service
 	RCPROG=service
 	INITSCRIPT=laptop-mode
 else
 	# Any other -- we start it ourselves.
 	RCPROG=
-	INITSCRIPT=/etc/init.d/laptop-mode
+	INITSCRIPT=$INIT_D/laptop-mode
 fi
 
 if ( ! $RCPROG $INITSCRIPT start ) ; then
